#include <stdlib.h>
#include <gtk/gtk.h>

#include <libchroma/chroma-list.h>
#include <libchroma/chroma-list-column.h>
#include <libchroma/chroma-vscrollbar.h>

typedef struct {
    gchar *artist;
    gchar *album;
    gchar *title;
} TrackinfoTuple;

static TrackinfoTuple title_names[] = {
    {"Portishead" ,"Dummy", "Mysterons"},
    {"Portishead" ,"Dummy", "Sour Times"},
    {"Portishead" ,"Dummy", "Strangers"},
    {"Portishead" ,"Dummy", "It Could Be Sweet"},
    {"Portishead" ,"Dummy", "Wandering Star"},
    {"Portishead" ,"Dummy", "It's A Fire"},
    {"Portishead" ,"Dummy", "Numb"},
    {"Portishead" ,"Dummy", "Roads"},
    {"Portishead" ,"Dummy", "Pedestal"},
    {"Portishead" ,"Dummy", "Biscuit"},
    {"Portishead" ,"Dummy", "Glory Box"},
    {"Portishead" ,"Portishead", "Cowboys"},
    {"Portishead" ,"Portishead", "All Mine"},
    {"Portishead" ,"Portishead", "Undenied"},
    {"Portishead" ,"Portishead", "Half Day Closing"},
    {"Portishead" ,"Portishead", "Over"},
    {"Portishead" ,"Portishead", "Humming"},
    {"Portishead" ,"Portishead", "Mourning Air"},
    {"Portishead" ,"Portishead", "Seven Months"},
    {"Portishead" ,"Portishead", "Only You"},
    {"Portishead" ,"Portishead", "Elysium"},
    {"Portishead" ,"Portishead", "Western Eyes"}
};

static void 
time_cell_data_func (ChromaListColumn *self, GValue *value)
{
    gint time, minutes, seconds;
    gchar *time_str;

    time = g_value_get_int (value);

    g_value_unset (value);
    g_value_init (value, G_TYPE_STRING);

    minutes = time / 60;
    seconds = time % 60;

    time_str =g_strdup_printf("%d:%-2.2d",minutes,seconds); 

    g_value_set_string (value, time_str);
    g_free (time_str);
}


static gboolean 
on_list_configure            (GtkWidget *widget,
			      GdkEventConfigure *event,
                              gpointer user_data)
{
    gint total_width, row_height, reserved_space;
    gdouble grip_size;

    ChromaList *list = CHROMA_LIST(widget);
    ChromaVScrollbar *scrollbar = CHROMA_VSCROLLBAR (user_data);

    g_object_get (G_OBJECT(list), "row_height", &row_height, "reserved_space", &reserved_space, NULL);

    total_width = event->width - 60 - reserved_space; 
    
    chroma_list_set_column_width (list, 0, total_width/3); 
    chroma_list_set_column_width (list, 1, total_width/3);
    chroma_list_set_column_width (list, 2, total_width/3); 
    chroma_list_set_column_width (list, 3, 60); 

    gtk_widget_set_size_request (GTK_WIDGET(scrollbar), 14, event->height);

    return FALSE;
}

gint main (gint argc, gchar **argv)
{
    ChromaVScrollbar *scrollbar;
    ChromaListColumn *list_column;
    GtkWidget *window, *list, *hbox;
    GtkListStore *store;
    GtkTreeIter iter;
    gint n, ctr = 1;

    gtk_init (&argc, &argv);

    hbox = gtk_hbox_new (FALSE, 0);

    scrollbar = chroma_vscrollbar_new ();
    g_object_set (G_OBJECT(scrollbar), "slider_size", 0.5, NULL);

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);

    list = chroma_list_new ();

    store = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT);
    for (n = 0; n < G_N_ELEMENTS(title_names); n++)
      {
	    TrackinfoTuple info_tuple = title_names[n];
	    gtk_list_store_append (GTK_LIST_STORE(store), &iter);
	    gtk_list_store_set (GTK_LIST_STORE(store), &iter, 
				0, info_tuple.artist, 
				1, info_tuple.album, 
				2, info_tuple.title, 
				3, (gint)g_random_int_range(0,600), -1); 
      }

    chroma_list_set_model (list, store);
    chroma_list_set_column_cell_data_func (list, 3, time_cell_data_func);

    chroma_list_set_column_xalign (list, 0, 0.0);
    chroma_list_set_column_xalign (list, 1, 0.0);
    chroma_list_set_column_xalign (list, 2, 0.0);
    chroma_list_set_column_xalign (list, 3, 1.0);

    chroma_list_set_adjustment (list, scrollbar);
    
    g_object_connect (G_OBJECT(list), "signal::configure-event", G_CALLBACK(on_list_configure), scrollbar, NULL);

    gtk_box_pack_start (GTK_BOX(hbox), list, TRUE, TRUE, 0);
    gtk_box_pack_start (GTK_BOX(hbox), scrollbar, FALSE, FALSE, 0);

    gtk_container_add (GTK_CONTAINER (window), hbox);

    gtk_window_set_position (GTK_WINDOW (window), GTK_WIN_POS_CENTER);

    gtk_widget_show_all (window);
    
    gtk_main ();

    exit (0);
}
